package net.sourceforge.nattable.painter.cell;

import net.sourceforge.nattable.config.IConfigRegistry;
import net.sourceforge.nattable.layer.ILayer;
import net.sourceforge.nattable.layer.cell.LayerCell;
import net.sourceforge.nattable.painter.cell.AbstractTextPainter;
import net.sourceforge.nattable.resize.command.ColumnResizeCommand;
import net.sourceforge.nattable.resize.command.RowResizeCommand;
import net.sourceforge.nattable.style.CellStyleUtil;
import net.sourceforge.nattable.style.IStyle;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Rectangle;

/**
 * TextPainter that draws text into a cell vertically.
 * Can handle word wrapping and/or word cutting and/or automatic calculation and resizing of the
 * cell width and height if the text does not fit into the cell.
 */
public class VerticalTextPainter extends AbstractTextPainter {

	public VerticalTextPainter() {
		this(false, true);
	}

	/**
	 * @param wrapText split text over multiple lines
	 * @param paintBg skips painting the background if is FALSE
	 */
	public VerticalTextPainter(boolean wrapText, boolean paintBg) {
		this(wrapText, paintBg, 0);
	}
	
	/**
	 * @param wrapText split text over multiple lines
	 * @param paintBg skips painting the background if is FALSE
	 * @param spacing
	 */
	public VerticalTextPainter(boolean wrapText, boolean paintBg, int spacing) {
		this(wrapText, paintBg, spacing, false);
	}

	
	/**
	 * @param wrapText split text over multiple lines
	 * @param paintBg skips painting the background if is FALSE
	 * @param spacing
	 * @param calculateColumnHeight automatically calculate the height of the column
	 * corresponding to the longest text value within that row
	 */
	public VerticalTextPainter(boolean wrapText, boolean paintBg, int spacing, boolean calculate) {
		super(wrapText, paintBg, spacing, calculate);
	}

	@Override
	public int getPreferredWidth(LayerCell cell, GC gc, IConfigRegistry configRegistry){
		setupGCFromConfig(gc, CellStyleUtil.getCellStyle(cell, configRegistry));
		return gc.textExtent(convertDataType(cell, configRegistry)).x;
	}

	@Override
	public int getPreferredHeight(LayerCell cell, GC gc, IConfigRegistry configRegistry) {
		setupGCFromConfig(gc, CellStyleUtil.getCellStyle(cell, configRegistry));
		return getLengthFromCache(gc, convertDataType(cell, configRegistry)) + (spacing*2) + 1;
	}

	@Override
	public void paintCell(LayerCell cell, GC gc, Rectangle rectangle, IConfigRegistry configRegistry) {
		if (paintBg) {
			super.paintCell(cell, gc, rectangle, configRegistry);
		}

		Rectangle originalClipping = gc.getClipping();
		gc.setClipping(rectangle.intersection(originalClipping));

		IStyle cellStyle = CellStyleUtil.getCellStyle(cell, configRegistry);
		setupGCFromConfig(gc, cellStyle);
		String text = convertDataType(cell, configRegistry);

		//calculate the text to display, adds dots if the text is longer than the available
		//row height and adds new lines instead of spaces if word wrapping is enabled
		text = getTextToDisplay(cell, gc, rectangle.height, text);

		int numberOfNewLines = getNumberOfNewLines(text);
		
		//if the content width is bigger than the available column width
		//we're extending the column width (only if word wrapping is enabled)
		int fontHeight = gc.getFontMetrics().getHeight();
		int contentWidth = fontHeight * numberOfNewLines;
		int contentToCellDiff = (cell.getBounds().width - rectangle.width) * numberOfNewLines;
		
		if ((contentWidth > rectangle.width) && wrapText) {
			ILayer layer = cell.getLayer();
			layer.doCommand(
					new ColumnResizeCommand(
							layer, 
							cell.getColumnPosition(), 
							contentWidth + (spacing*2) + contentToCellDiff));
		}
		
		if (text != null && text.length() > 0) {
			if (numberOfNewLines == 1) {
				int contentHeight = Math.min(getLengthFromCache(gc, text), rectangle.height);
				
				GraphicsUtils.drawVerticalText(
						text, 
						rectangle.x + CellStyleUtil.getHorizontalAlignmentPadding(cellStyle, rectangle, contentWidth),
						rectangle.y + CellStyleUtil.getVerticalAlignmentPadding(cellStyle, rectangle, contentHeight + spacing),
						gc, 
						SWT.UP);
			}
			else {
				//draw every line by itself because of the alignment, otherwise the whole text
				//is always aligned right
				int xStartPos = rectangle.x + CellStyleUtil.getHorizontalAlignmentPadding(cellStyle, rectangle, contentWidth);
				String[] lines = text.split("\n");
				for (String line : lines) {
					int lineContentWidth = Math.min(getLengthFromCache(gc, line), rectangle.width);
					
					GraphicsUtils.drawVerticalText(
							line,
							xStartPos,
							rectangle.y + CellStyleUtil.getVerticalAlignmentPadding(cellStyle, rectangle, lineContentWidth + spacing),
							gc,
							SWT.UP);
					
					//after every line calculate the y start pos new
					xStartPos += fontHeight;
				}
			}
		}

		gc.setClipping(originalClipping);
	}

	@Override
	protected void setNewMinLength(LayerCell cell, int contentHeight) {
		int cellLength = cell.getBounds().height;
		if (cellLength < contentHeight) {

			ILayer layer = cell.getLayer();
			int row = layer.getRowIndexByPosition(cell.getRowPosition());
			layer.doCommand(new RowResizeCommand(layer, row, 
					contentHeight));
		}
	}

	@Override
	protected int calculatePadding(LayerCell cell, int availableLength) {
		return cell.getBounds().height - availableLength;
	}
}