package net.sourceforge.nattable.reorder.action;

import net.sourceforge.nattable.NatTable;
import net.sourceforge.nattable.layer.ILayer;
import net.sourceforge.nattable.layer.cell.LayerCell;
import net.sourceforge.nattable.painter.IOverlayPainter;
import net.sourceforge.nattable.reorder.command.ColumnReorderCommand;
import net.sourceforge.nattable.selection.command.ClearAllSelectionsCommand;
import net.sourceforge.nattable.ui.action.IDragMode;
import net.sourceforge.nattable.ui.util.CellEdgeDetectUtil;
import net.sourceforge.nattable.ui.util.CellEdgeEnum;
import net.sourceforge.nattable.util.GUIHelper;

import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;

/**
 * Default {@link IDragMode} invoked for 'left click + drag' on the column header.<br/>
 * It does the following when invoked: <br/>
 * <ol>
 *    <li>Fires a column reorder command, to move columns</li>
 *    <li>Overlays a black line indicating the new column position</li>
 * </ol>
 */
public class ColumnReorderDragMode implements IDragMode {
	
	private NatTable natTable;
	private MouseEvent initialEvent;
	private MouseEvent currentEvent;
	
	protected ColumnReorderOverlayPainter targetOverlayPainter = new ColumnReorderOverlayPainter();
	
	public void mouseDown(NatTable natTable, MouseEvent event) {
		this.natTable = natTable;
		initialEvent = event;
		currentEvent = initialEvent;
		
        natTable.addOverlayPainter(targetOverlayPainter);
        
        natTable.doCommand(new ClearAllSelectionsCommand());
	}
	
	public void mouseMove(NatTable natTable, MouseEvent event) {
		currentEvent = event;
		
		natTable.redraw(0, 0, natTable.getWidth(), natTable.getHeight(), false);
	}

	public void mouseUp(NatTable natTable, MouseEvent event) {
		natTable.removeOverlayPainter(targetOverlayPainter);
		
		int dragFromGridColumnPosition = getDragFromGridColumnPosition();
		int dragToGridColumnPosition = getDragToGridColumnPosition(getMoveDirection(event.x), natTable.getColumnPositionByX(event.x));
		
		if (isValidTargetColumnPosition(natTable, dragFromGridColumnPosition, dragToGridColumnPosition, event)) {
			fireMoveCommand(natTable, dragFromGridColumnPosition, dragToGridColumnPosition);
		}
		
		natTable.redraw(0, 0, natTable.getWidth(), natTable.getHeight(), false);
	}
	
	private int getDragFromGridColumnPosition() {
		return natTable.getColumnPositionByX(initialEvent.x);
	}
	
	private int getDragToGridColumnPosition(CellEdgeEnum moveDirection, int gridColumnPosition) {
		int dragToGridColumnPosition = -1;
		
		if (moveDirection != null) {
			switch (moveDirection) {
			case LEFT:
				dragToGridColumnPosition = gridColumnPosition;
				break;
			case RIGHT:
				dragToGridColumnPosition = gridColumnPosition + 1;
				break;
			}
		}
		
		return dragToGridColumnPosition;
	}
	
	private CellEdgeEnum getMoveDirection(int x) {
	    LayerCell cell = getColumnCell(x);
	    if (cell != null) {
			Rectangle selectedColumnHeaderRect = cell.getBounds();
			return CellEdgeDetectUtil.getHorizontalCellEdge(selectedColumnHeaderRect, new Point(x, initialEvent.y));
	    }
		
		return null;
	}
	
	private LayerCell getColumnCell(int x) {
	    int gridColumnPosition = natTable.getColumnPositionByX(x);
	    int gridRowPosition = natTable.getRowPositionByY(initialEvent.y);
	    return natTable.getCellByPosition(gridColumnPosition, gridRowPosition);
	}

	protected boolean isValidTargetColumnPosition(ILayer natLayer, int dragFromGridColumnPosition, int dragToGridColumnPosition, MouseEvent event) {
		return dragFromGridColumnPosition >= 0 && dragToGridColumnPosition >= 0;
	}

	protected void fireMoveCommand(NatTable natTable, int dragFromGridColumnPosition, int dragToGridColumnPosition) {
		natTable.doCommand(new ColumnReorderCommand(natTable, dragFromGridColumnPosition, dragToGridColumnPosition));
	}

	private class ColumnReorderOverlayPainter implements IOverlayPainter {

		public void paintOverlay(GC gc, ILayer layer) {
			int dragFromGridColumnPosition = getDragFromGridColumnPosition();
			
			if (currentEvent.x > natTable.getWidth()) {
				return;
			}
			
			CellEdgeEnum moveDirection = getMoveDirection(currentEvent.x);
			int dragToGridColumnPosition = getDragToGridColumnPosition(moveDirection, natTable.getColumnPositionByX(currentEvent.x));
			
			if (isValidTargetColumnPosition(natTable, dragFromGridColumnPosition, dragToGridColumnPosition, currentEvent)) {
				int dragToColumnHandleX = -1;
				
				if (moveDirection != null) {
					Rectangle selectedColumnHeaderRect = getColumnCell(currentEvent.x).getBounds();
					
					switch (moveDirection) {
					case LEFT:
						dragToColumnHandleX = selectedColumnHeaderRect.x;
						break;
					case RIGHT:
						dragToColumnHandleX = selectedColumnHeaderRect.x + selectedColumnHeaderRect.width;
						break;
					}
				}
				
				if (dragToColumnHandleX > 0) {
					Color orgBgColor = gc.getBackground();
					gc.setBackground(GUIHelper.COLOR_DARK_GRAY);
					
					gc.fillRectangle(dragToColumnHandleX - 1, 0, 2, layer.getHeight());
					
					gc.setBackground(orgBgColor);
				}
			}
		}

	}
	
}
